
//#define debug

#define F_CPU 8000000ul
#include <util/delay.h>
#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdbool.h>
#include <avr/pgmspace.h>
#include "port_conf.h"
#include "adc.h"
#include "uart.h"
#include "timers.h"
#include "led.h"
#include "NMEA_rk.h"
#include <string.h>
#include <avr/wdt.h>


#include "fatfs/xitoa.h"
#include "fatfs/ff.h"
#include "fatfs/diskio.h"


#include "operacje_sd.h"



#define okres 10//okrela czas w sekundach  pomidzy kolejnymi pomiarami
#define Delay_time_1 1 //czas w minutach, po ktrym ukad przejdzie do upienia z trybu - rej.zatrzymana [ked
#define Delay_time_2 20//czas w minutach, po ktrym ukad przejdzie do upienia z trybu - rej.pauza
#define margines 60

enum tryb_pracy {rej_zatrzymana, rej_pauza, rej_praca};


extern volatile DSTATUS Stat;

static volatile char counter;
static volatile enum tryb_pracy tryb;
static volatile char space_time_min; 
static volatile char space_time_sec;
static volatile char space_time_tmp;
static volatile bool flag_int1;
volatile bool card_error;

extern struct GPRMC GP_RMC;

char current_file[12];
BYTE bufor1[512];
BYTE bufor_rmc[100];
char licznik_buforowania=0;
unsigned int old_lat, old_long;


//struktura ramki GPRMC
struct GPRMC
{
	char time[UTC_time_size+2];
	char Status;
	char Latitude[Latitude_size+2];
	//char NSIndicator;
	char Longtitude[Longtitude_size+2];
	//char EWIndicator;
	//char Speed[Speed_size+2];
	//char Course[Course_size+2]; 
	char Date[Date_size+2];
	//char MVariation[Mvariation_size+2];
	//char MVariation_indicator;
	//char Mode;
	char CRC[3];
	
	unsigned int fractional_lat;
	unsigned int fractional_long;
	
};



void konfiguracja_kontrolera(void)
{
 
   usart_init(uart_rate);
   //port_init();
   
   DDRA=0x83;
   PORTA=0x80;
	
   DDRB=0xB2;
   PORTB=0x1F;
	
   DDRD = 0x02;
   PORTD = 0x1C;
   
   //konfiguracja pinw od stanu baterii
   DDRA &= ~(1<<2 | 1<<1);
   PORTA |= (1<<2 | 1<<1);
   
   //konfiguracja ledow. bat
   DDRC |=( 1<<1 | 1<<2 | 1<<3);
   
   //POWER ON port
   DDRD |= 1<<6;
   
   
   device_on;
   gps_off;
   led_timer_init(1);
   fat_timer_init(1);
   cli();
   led1_on;
   led2_on;
   led3_on;
   _delay_ms(1000);
   _delay_ms(1000);
   led1_off;
   led2_off;
   led3_off;
   
   GIMSK = ( 1<<INT0 | 1<<INT1);
   sei();
}

//sprawdzanie stanu akumulatora
void check_battery(void)
{
	if ((PINA & (1<<2)) ==0)
	   {
		    if ((PINA & (1<<1)) ==0)
				led_display(false,fast_blink,dont_change); 
			else
				led_display(false,pulse,dont_change);
	   } else
		led_display(false,off,dont_change);
}


int main(void)
{
		
   konfiguracja_kontrolera();
   tryb=rej_zatrzymana;
   current_file[0]=0;
   
   space_time_tmp=0;
   space_time_sec=0;
   space_time_min=0;
   card_error=0;
	
	led_display(false,dont_change,slow_blink);
	

	
	
	
	while(1)
    {
		
		
		switch (tryb)
		{
		
		case rej_zatrzymana:
			
			led3_off;
			
			if ((current_file[0]!=0))
			{
				if (licznik_buforowania!=0)
				{
					licznik_buforowania =0;
					#ifdef debug
					usart_string("rej. zatrzymana.. zapisuje bufor do pliku");
					#endif
					append_to_file(current_file,bufor1,strlen(bufor1));
					bufor1[0]=0;
				}
			}
			
		 
			led_display(0,dont_change,slow_blink);
			gps_off;
			current_file[0]=0;
			licznik_buforowania=0;
			
			if (space_time_min >= Delay_time_1) uspij();
			break; 
			
		case rej_praca:
		    gps_on;
			space_time_min=0;
			
			 
 		    if (current_file[0]==0)	//aktualnie urzdzenie nie pracuje na adnym pliku
			{
				#ifdef debug
				usart_string("start...");
		        usart_string("szukam..");
				#endif
				
				new_file_name(current_file);
				current_file[0]=0;
				
				if (new_file_name(current_file)==1)  //szukanie nazwy dla nowego pliku
			    {
					card_error=0;
					led3_on;
					#ifdef debug
					usart_string("znaleziono ");
					usart_string(current_file);
					#endif
				} 
				else								 //proces szukania pliku zakonczony niepowodzeniem (pozycje obsadzone lub problem z SD)
				{
					#ifdef debug
					usart_string("ErrSD");
					#endif
					
					led_display(0,dont_change,three_flash);
					card_error=1;
					gps_off;
					
					while(tryb!=rej_zatrzymana){_delay_ms(100); gps_off;}
				}
			} 
			
			if (current_file[0]!=0)
			{
			led_display(0,dont_change,continous);
			card_error=0;
				
				
				if (GP_RMC.Status !='A') led3_on;
				
				
				
				
				if (space_time_sec >=okres) //prba dokonania kolejnego pomiaru
				{
					space_time_sec=0;
					if (licznik_buforowania ==0) bufor1[0]=0;
					
					while(tryb==rej_praca)
 					{
 						//-----------------------------pobiera ramk GPRMC i dokonuje sprawdzenia------------------------------
						if (get_RMC_frame(bufor_rmc)==1)
						{	
							divide_frame(bufor_rmc);
							
							if (GP_RMC.Status=='A')
							{
								get_fattime(0); //aktualizuje czas i date
								
								
								
 								if (((abs(old_long-GP_RMC.fractional_long))<margines) &&  ((abs(old_lat-GP_RMC.fractional_lat))<margines))
								{
									#ifdef  debug
									usart_string("NZAP");
									#endif
									}	
									else
 									{
									#ifdef  debug
									usart_string("incr licz");
									#endif 
									
									strcat(bufor1,bufor_rmc);
									
								   
									old_long=GP_RMC.fractional_long;
									old_lat=GP_RMC.fractional_lat;
									 
									licznik_buforowania++;	
									
									led3_on;
									_delay_ms(50);
									
									if (licznik_buforowania == 6)
									{
	
										
										
										licznik_buforowania=0;
										
										
										
										
										if ((append_to_file(current_file,bufor1,strlen(bufor1)))==0)
	 									{
											f_mount(0, 0);
		 									led_display(0,dont_change,three_flash);
		  									card_error=1;
 									 
		 								    while(tryb!=rej_zatrzymana){_delay_ms(100); gps_off; led3_off;}
		  								}	
										  
										_delay_ms(1000);
										led3_off;
										break;
									} else
									{
										led3_off;
										break;	
									}
									
								}									 
								
								
								
							} else 
							{
								#ifdef debug
								usart_string("brak fix");
								#endif
								
								led3_on;
								
								break;
							}								
						}							
						
					    //bufor1[0]=0;	
					}						
					
				}
		    }
			break;
		
		case rej_pauza:
		
			led3_off;
			
		    if (licznik_buforowania != 0)
			{
				licznik_buforowania =0;
				append_to_file(current_file,bufor1,strlen(bufor1));
				bufor1[0]=0;
			}
			
		    led_display(0,dont_change,fast_blink);
			gps_off;
			
			if (space_time_min >= Delay_time_2) 
			{
				#ifdef debug
					usart_string("zamykam_ z pauzy");
				#endif
				uspij();
			}
			break;
		}
	}
}

ISR (TIMER0_OVF_vect) //przerwanie 16Hz
{
	bufor1[511]=0;
	wdt_reset();
	led_display(1,0,0);
	space_time_tmp++;
	if (space_time_tmp>30) {space_time_tmp=0; space_time_sec++; check_battery();} //inkrementowanie co 1s
	
	if (space_time_sec>=60) {space_time_sec=0; space_time_min++;} 
	
	if ((PIND & 1<<3) !=0) flag_int1=0;
} 

ISR (TIMER2_COMP_vect) //przerwanie 100Hz dla  fatfs
{
	wdt_reset();
	TCNT2=0;
	disk_timerproc();	/* Drive timer procedure of low level disk I/O module */
} 



ISR (INT1_vect)
{
	
	wdt_reset();
	//led_display(true,dont_change,dont_change);

	
	int cnt=0,cnt1=0;
	space_time_min=0;
	space_time_sec=0;
	
	if (card_error==1) 
	{
		#ifdef debug
			usart_string("kasuje card err");
		#endif
		card_error=0;
		tryb=rej_zatrzymana;
		_delay_ms(1000);
		return;
	}

 	
 	if (flag_int1==0)
 	{
 		flag_int1=1;
		while(1)
		{
			cnt1++;
			if (cnt1 == 4)
			{
				led_display(true,0,0);
				cnt1=0;
			}				
			wdt_reset();
			
			
			_delay_ms(10);
			if (cnt < 150)
			{
				if ((PIND & (1<<3)) !=0) break;
				cnt++;	
			}
			else
			{
				tryb=rej_zatrzymana;
				if ((PIND & (1<<3)) !=0) return;
			}
			
			
		}
		
		switch(tryb)
		{
			case rej_zatrzymana:
				tryb=rej_praca;
				break;
			case rej_praca:
				tryb=rej_pauza;
				break;
			case rej_pauza:
				tryb=rej_praca;
				break;
		}
	}
}

ISR(INT0_vect)
{
	if (tryb==rej_zatrzymana) 
	{
		cli();
		uspij();
	}		
}



//konfiguruje ukd w tryb najmniejszego poboru mocy
void uspij(void)
{
    #ifdef debug
	usart_string("usypianie");
	#endif
	
	cli();
	wdt_disable();
	gps_off;
 	
	
	led1_on;
	led2_on;
	led3_on;
	
	_delay_ms(1000); 
	_delay_ms(1000); 
	
	
    PORTD |= 1<<6;
	
	

	
	while(1)
	{
	    DDRD=0;
		PORTD=1<<7;
		wdt_reset();
	}		
	 
	//zaswiec wszystkie ledy na chwile 2s
	
	//odcz zasilanie ukadu

}

void flash2buffer(const char *data,char* buffer) 
{ 
    while (pgm_read_byte(data) != 0x00) 
    *(buffer++)=(pgm_read_byte(data++)); 
	*buffer=0;
} 








